from helpers import Helper
from logger import Logger
logger = Logger.getInstance()

class Text:
    
    def palindromic_partitions(_str:str):
        pass
    def text_to_str_list(_text: str,):
        # convert a text of words into a list of words
        return _text.split(" ")    


class Word:
    # a word is generic container for text  type that has multiple usefull representations 
    # for example  text_rep = "word", list_rep = ['w',o','r','d']
    # so word object will containe all needed representation of a string for specific task 
    ##### Implementation: 
    # take a string "cat" --------store/convert it-------> ['c','a','t']
    def __init__(self,_str:str):
        self._str = _str
        self._list = list(_str)
    def __str__(self):
        return str((self._str,self._list))
from g2p_en import G2p
g2p = G2p()
class Sound(Word):
    def __init__(self,_str):
        # find g2p sound represintation
        # 
        #
        # 
        #  
        #g2p = G2p()
        #super().__init__(_str)
        
        self._str = _str
        self._list = list(_str) 
        self._sound = g2p(_str)
        self._sound_str = "_".join(self._sound)
    def __str__(self):
        return str((self._str,self._list,self._sound,self._sound_str))

    def __repr__(self):
        return str((self._str,self._list,self._sound,self._sound_str))

    def __len__(self):
        return len(self._sound)
    def __eq__(self,other):
        return self._sound == other._sound
    def __hash__(self):
        #return self._sound_str
        return hash(self._sound_str)
    
    
def list_to_counted_dict(_list:list):
  _dict ={}
  for k in _list:
    _dict[k] = _dict.get(k,0)  + 1
  return _dict


def similarity_weight(word1,word2):
    # find every subsets of the first word 
    # find every subsets of the second word
    # store all subsets that are in both word1_subsets and word2_subsets with their count .
    ### calculate the count => count = count_in_word1_subsets * count_in_word2_subsets.   
    # return similarity_weight  = the intersected_subsets with their count
    #  
    word1_subsets = Helper.permutations(tuple(word1._sound))
    word2_subsets = Helper.permutations(tuple(word2._sound))
    subsets1_count = list_to_counted_dict(word1_subsets)
    subsets2_count = list_to_counted_dict(word2_subsets)
    
    
    _similarity_weight = {}

    for subset_key in subsets1_count:
        
        _similarity_weight[subset_key] = subsets1_count[subset_key] * subsets2_count.get(subset_key,0)

    is_zero = lambda x : x[1] > 0
    _similarity_weight = dict(filter(is_zero,_similarity_weight.items()))

    return _similarity_weight

def weight_len_product (weight):
    _weight_len_product = {}
    for k,v in weight.items():
        _weight_len_product[k] = len(k) * v
    return _weight_len_product
if __name__ == "__main__":
    # ## test Word class ##
    # _str = "hello"
    # word = Word(_str)
    # print("word: \n", word)
    # #####################


    # ## test Text class ##
    # _str = "hello Text this is a long text"
    # print("text to words: \n",Text.text_to_str_list(_str))
    # Text.palindromic_partitions(_str)
    # #####################

    # ## test Sound class ##
    
    # # make a long text made of words
    # # convert text to list of strings 
    # # from the list of strings create a list of sounds
    # #   
    # sound = Sound("sound")
    # print("sound:\n", sound)

    # str_list = Text.text_to_str_list("time to hear the sounds of these words")
    # sounds_list = list(map(Sound,str_list)) 
    
    # print(*sounds_list,sep='\n')
    
    # ######################

    # ## test similarity_weight() ##
    # from sort import Sort

    # sound1 = Sound('cat hat')
    # sound2 = Sound('fat rat')
    # print("sound1:\n",sound1)
    # print("sound2:\n",sound2)

    # weight = similarity_weight(sound1,sound2)
    # _weight_len_product = weight_len_product(weight)
    # sort_weight_len_product = Sort.sort_dict_by_value(_weight_len_product)

    # print("weight:\n",weight)
    # print("sorted weight:\n",Sort.sort_dict_by_value(weight))
    # print("sorted weight_len_product:\n",sort_weight_len_product)

    # ######################

    # ## Test hashable sound ##
    # import pprint
    # sound1 = Sound('drink')
    # _dict = {}
    # _dict[sound1] = 5
    # _dict[Sound('mat')] = 3
    
    # #print("_dict:\n",_dict)
    # pprint.pprint(_dict)
    
    # ##########################

    ## Test save data to csv ##
    import pandas as pd
    
    data = {'col_1': [3, 2, 1, 0], 'col_2': ['a', 'b', 'c', 'd']}
    # data = {'a':1,'b':2,'c':3}
    # pd.DataFrame(d.items(), columns=['Date', 'DateValue'])
    data_frame = pd.DataFrame(data.items(), columns=['key', 'value'])
    print(data_frame)
    # exit()
    
    sound1 = Sound('drink')
    sound2 = Sound('good')
    
    _dict = {}
    _dict[sound1] = 5
    _dict[Sound('mat')] = 342
    
    #_dict2 = {sound2:1,'b':2,'c':sound1}
    # data_frame = pd.DataFrame(_dict2)
    #data_frame = pd.DataFrame(_dict2.items(),columns=['word','count'])
    
    data_frame = pd.DataFrame(_dict.items(),columns=['sound','count'])
    # print("type(data_frame['word'][0]:",type(data_frame['word'][0]))
    
    print(data_frame)
    Helper.save_data_frame_to_csv("sound_count_data.csv",data_frame)

    ###########################

    
    
 



from random import choices

import logger

class Sort:
    
    def sort_dict_by_value(_dict: dict,reverse = True):
        
        _dict = {k:v for k,v in sorted(_dict.items(),key = lambda item:item[1],reverse = reverse)}
        return _dict
if __name__ == "__main__":
    logger = logger.Logger.getInstance()
    _dict  = {"a":5 ,'h': 33,"b": 3, "c": 2}
    
    sorted_dict = Sort.sort_dict_by_value(_dict, False)
    # _dict['g'] = 15
    # sorted_dict['d'] = 10
    # population = [1,2,3,4,5,6,7]
    #weights = [0.1, 0.05, 0.05, 0.2, 0.4, 0.2,0.9]
    population = [3,7]
    
    weights = [0.1,0.1]

    logger.log("_dict: %s" % _dict)
    logger.log("sorted_dict: %s" % sorted_dict)
    
    million_samples = choices(population, weights, k=10**6)
    from collections import Counter
    
    _count = Counter(million_samples)
    print(type(_count))
    total = sum(_count)
    print(sum(_count))
    logger.log("total: %s" % total)
    normalized = list(map(lambda x: x/total, _count))
    logger.log("count probableity distibution: %s" % _count )
    logger.log("normalizedprobableity distibution: %s" % normalized )

    #logger.log("probailatiy distribution: %s" % choices(population,weights,k=10))





import pandas as pd

class Helper:
    #segment is the function that divied the alphabets 
    def permutations(alphabets,segment=lambda x: x,_join=lambda x: x):
         #the defualt segmentation function is to itorate over chars in words
        perms : list = [] #  list of all permutations
        #print("alphabets before segment: %s" % alphabets)
        segmented_alphabets = segment(alphabets)
        #print("segmented_alphabets: %s" % segmented_alphabets)

        n = len(segmented_alphabets)# number of elements 
        #arr = []
        for i in range(0,n):# i is save point 

            for j in range(i,n):# j is the steps away from the save point 
                perms.append(segmented_alphabets[i:j+1])
        
        #print("perms: %s" % perms)

        #_join = lambda x: "_".join(x)
        #reverse the split operation, becuase we need hashable keys not arrays 
        perms = list(map(_join,perms))
        #print("perms: %s" % perms)

        return perms
#the name dictionary does refer to a language dictionary as an analogy,
#and not to python's dict data type 
#dictionary is list of words
#word is list of alphabets/characters
    def permutations_on_2dlist(dictionary):
        flat_perms: list = [] # flat means all elements have been appended to one 1D list
        
        for word in dictionary:#
            flat_perms.extend(Helper.permutations(word))
        return flat_perms

#get the input to DataFrame
    # import itertools
    # def data_frame_input(keys: tuple,values: tuple):
    #     for k,v in zip((keys,values)):
    #         print("k: %s,v: %s" % (k,v))


    def data_frame_input(keys: list,values: list):
    #def data_frame_input(keys_values: tuple):
    
        _dict = {}
        #keys = keys_values[0]
        #values = keys_values[1]
        for k,v in zip(keys,values):
            
            _dict[k] = v

        print("_dict: ", _dict)
        return _dict
    #df_input : data frame input
    #df : data frame
    def create_data_frame(keys: list, values: list):
        #data frame input
        df_input = Helper.data_frame_input(keys,values)
        df = pd.DataFrame(df_input)
        
        return df

    def save_data_frame_to_csv(file_path,data_frame):
        data_frame.to_csv(file_path)

    def convert_list_to_dict(_list: list,init_value):
        _dict = {k: init_value for k in _list}
        
        return _dict
    def score_permutation_by_word(permutations_score: dict,word,word_score,segment=lambda x: x,_join=lambda x: x):
       
        word_permutations: list = Helper.permutations(word,segment,_join)
        #print("word_permutations: %s" % word_permutations )
        for perm in word_permutations:
            #print(perm,end=' ')
            #print(word)
            permutations_score[perm] = permutations_score.get(perm,0) + word_score    
        return permutations_score
    
    # def _map_score_permutation_by_word(_dict: dict):
    #     return Helper.score_permutation_by_word(_dict['permutations_score'],
    #                                     _dict['word'],
    #                                     _dict['word_score'])
    


    def score_permutations_by_words(permutations_score: dict,words,words_score,segment=lambda x: x,_join=lambda x: x):
    #def score_permutations_words(_dict: dict):
        #map(Helper._map_score_permutation_by_word,zip(_dict))
        
        for word,word_score in zip(words,words_score):
            Helper.score_permutation_by_word(permutations_score, word, word_score,segment,_join)
        return permutations_score  

    #dict_keys =  dict.keys()
    #condition is a user defined function that be used as a filtering criteria  
    def get_alphabet_from_dictionary(condition ,dict_keys: list):
        alphabet: list = list (filter(condition,dict_keys))
        
        return alphabet

if __name__ == "__main__":

    keys = [1,2,3,4]
    values = ['a','b','c','d']
    
    
    _dict = Helper.convert_list_to_dict(keys,0)
    print(_dict)
    
    _dict = Helper.convert_list_to_dict(values,'z')
    print(_dict)
    #df = pd.DataFrame(data_frame_input)
    

    df = Helper.create_data_frame(['Keys','Names'],[keys,values])
    print("df: \n",df)
    Helper.save_data_frame_to_csv("data_1.csv",df)
    
    #words = ["time"]
    #freqs = [1000]
    # words: list = "time to do this rrrignt abdullah".split(" ")
    # freqs: list = [100] * 6
    # perms_score_dict = {}
    # #Helper.score_permutation_by_word(perms_score_dict,words[4],freqs[4])
    # #print(perms_score_dict)
    # #Helper.score_permutations_words({'permutations_score': perms_score_dict, 'word': words,'word_score': freqs })
    

    # Helper.score_permutations_by_words(perms_score_dict, words,freqs)

    sounds = [['W'], ['W', 'ER1'], ['W', 'ER1', 'D'], ['W', 'ER1', 'D', 'B'], ['W', 'ER1', 'D', 'B', 'R'], ['W', 'ER1', 'D', 'B', 'R', 'IY2'], ['W', 'ER1', 'D', 'B', 'R', 'IY2', 'K'], ['W', 'ER1', 'D', 'B', 'R', 'IY2', 'K', 'ER0'], ['ER1'], ['ER1', 'D'], ['ER1', 'D', 'B'], ['ER1', 'D', 'B', 'R'], ['ER1', 'D', 'B', 'R', 'IY2'], ['ER1', 'D', 'B', 'R', 'IY2', 'K'], ['ER1', 'D', 'B', 'R', 'IY2', 'K', 'ER0'], ['D'], ['D', 'B'], ['D', 'B', 'R'], ['D', 'B', 'R', 'IY2'], ['D', 'B', 'R', 'IY2', 'K'], ['D', 'B', 'R', 'IY2', 'K', 'ER0'], ['B'], ['B', 'R'], ['B', 'R', 'IY2'], ['B', 'R', 'IY2', 'K'], ['B', 'R', 'IY2', 'K', 'ER0'], ['R'], ['R', 'IY2'], ['R', 'IY2', 'K'], ['R', 'IY2', 'K', 'ER0'], ['IY2'], ['IY2', 'K'], ['IY2', 'K', 'ER0'], ['K'], ['K', 'ER0'], ['ER0']]
    freqs = [1] * len(sounds)
    perms_score_dict = {}
    hashable_sounds = list(map("_".join,sounds))
    print("hashable_sounds: %s" % hashable_sounds)
    segment = lambda x: x.split("_")
    join = lambda x: "_".join(x)
    print("join: ",list(map(join,sounds)))
    #exit()
    #segment = lambda x: x
    print("segment_sounds: %s" % list(map(segment,hashable_sounds)))
    
    # exit()
    Helper.score_permutations_by_words(perms_score_dict,hashable_sounds,freqs,segment,join)
    



    print(perms_score_dict)

    dict_keys = perms_score_dict.keys()
    print("dict_keys: \n", dict_keys)
    is_alphabet = lambda k: len(k) == 1 
    alphabet = Helper.get_alphabet_from_dictionary(is_alphabet, dict_keys)
    print("alphabet: \n",alphabet)